#!/usr/bin/env node

const fs = require('fs');
const createConstants = require('./redux/constants');
const createActions = require('./redux/actions');
const createReducers = require('./redux/reducers');
const createSelector = require('./redux/selectors');
const { createSagaFunc, createSagaWatcher, createRootSaga } = require('./redux/sagas');

const params = process.argv.slice(2);
if (params.length < 3) {
  console.log('Eksik parametre girdiniz.');
} else {
  const mainName = params[0];
  const actionParams = params[1];
  const reducerState = params[2];

  const checkFile = fs.readFileSync('./src/store/constants.js').toString().split("\n");
  if (!checkFile.some(line => line.includes(`GET_${mainName.toUpperCase()}`) || line.includes(`SET_${mainName.toUpperCase()}`))) {
    const constants = createConstants(mainName);
    const actions = createActions(mainName, actionParams);
    const reducers = createReducers(mainName, reducerState);
    const sagaFunc = createSagaFunc(mainName, actionParams);
    const sagaWatcher = createSagaWatcher(mainName, actionParams);
    const rootSaga = createRootSaga(mainName);
    const selectors = createSelector(mainName, reducerState);
  
    fs.appendFile('./src/store/constants.js', constants, function (err) {
      if (err) throw err;
      console.log('Constants Created!');
    });
    fs.appendFile('./src/store/actions.js', actions, function (err) {
      if (err) throw err;
      console.log('Actions Created!');
    });
    fs.appendFile('./src/store/selectors.js', selectors, function (err) {
      if (err) throw err;
      console.log('Selectors Created!');
    });
    
    var sagaFile = fs.readFileSync('./src/store/sagas.js').toString().split("\n");
    // add saga func and watcher
    const lineNumberRootSaga = sagaFile.findIndex((item) => item.includes('yield all(['));
    sagaFile.splice(lineNumberRootSaga + 1, 0, rootSaga);
    // add watcher to rootSaga
    const lineNumberSagaFuncEnd = sagaFile.findIndex((item) => item.includes('Watcher()'));
    sagaFile.splice(lineNumberSagaFuncEnd, 0, sagaFunc + sagaWatcher);
    
    var sagaText = sagaFile.join("\n");
    fs.writeFile('./src/store/sagas.js', sagaText, function (err) {
      if (err) return console.log(err);
      console.log('Sagas Created!');
    });
    
    var reducerFile = fs.readFileSync('./src/store/reducers.js').toString().split("\n");
    const lineNumberReducerEnd = reducerFile.findIndex((item) => item.includes('default: {'));
    reducerFile.splice(lineNumberReducerEnd, 0, reducers);
    
    var reducersText = reducerFile.join("\n");
    fs.writeFile('./src/store/reducers.js', reducersText, function (err) {
      if (err) return console.log(err);
      console.log('Reducers Created!');
    });
  } else {
    console.log(`ilk parametre '${mainName.toUpperCase()}' benzersiz değil!`);
  }
}






